%       SUBROUTINE TDFRPH (IDOOD,FREQ,PHASE)
% *+
% *  - - - - - - - - - - -
% *   T D F R P H 
% *  - - - - - - - - - - -
% *
% *  This routine is part of the International Earth Rotation and
% *  Reference Systems Service (IERS) Conventions software collection.
% *
% *  This subroutine returns the frequency and phase of a tidal
% *  constituent when its Doodson number is given as input. 
% *
%%
%   Coded for VieVS: 
%   15 Jul 2011 by Hana Spicakova
%   26 Aug 2011 by Matthias Madzak, moved and simplified DD=zeros(1,6) --> DD = D;

function [FREQ PHASE] = libiers_tdfrph(IDOOD,mjd,leap)


%% changed according to VieVS
mjd_ref = 51544.5;       % 2000-01-01 at 12UT
T = (mjd - mjd_ref + (leap+32.184)/86400)/36525; 

DAYFR=(mjd-fix(mjd));  % [day]

%%

% * IERS expressions for the Delaunay arguments, in degrees

F1 =     134.9634025100D0 + ...
 T*( 477198.8675605000D0 + ...
 T*(      0.0088553333D0 + ...
 T*(      0.0000143431D0 + ...
 T*(     -0.0000000680D0 ))));
F2 =     357.5291091806D0 + ...
 T*(  35999.0502911389D0 + ... 
 T*(     -0.0001536667D0 + ...
 T*(      0.0000000378D0 + ...
 T*(     -0.0000000032D0 ))));
F3 =      93.2720906200D0 + ...
 T*( 483202.0174577222D0 + ...
 T*(     -0.0035420000D0 + ...
 T*(     -0.0000002881D0 + ...
 T*(      0.0000000012D0 ))));
F4 =     297.8501954694D0 + ...
 T*( 445267.1114469445D0 + ...
 T*(     -0.0017696111D0 + ...
 T*(      0.0000018314D0 + ...
 T*(     -0.0000000088D0 ))));
F5 =     125.0445550100D0 + ...
 T*(  -1934.1362619722D0 + ...
 T*(      0.0020756111D0 + ...
 T*(      0.0000021394D0 + ...
 T*(     -0.0000000165D0 ))));

% *  Convert to Doodson (Darwin) variables

D = zeros(1,6);
DD = D; % moved Matthias Madzak
D(1) = 360*DAYFR - F4; % HOUR ANGLE OF THE MOON
D(2) = F3 + F5; % MEAN LONGITUDE OF THE MOON
D(3) = D(2) - F4; % MEAN LONGITUDE OF THE SUN
D(4) = D(2) - F1; % MEAN LONGITUDE OF LUNAR PERIGEE
D(5) = -F5; % MEAN LONGITUDE OF THE ASCENDING NODE OF LUNAR ORBIT
D(6) = D(3) - F2; % MEAN LONGITUDE OF SOLAR PERIGEE

% *  Find frequencies of Delauney variables (in cycles/day), and from these
% *  the same for the Doodson arguments

FD1 =  0.0362916471D0 + 0.0000000013D0*T;
FD2 =  0.0027377786D0;
FD3 =  0.0367481951D0 - 0.0000000005D0*T;
FD4 =  0.0338631920D0 - 0.0000000003D0*T;
FD5 = -0.0001470938D0 + 0.0000000003D0*T;
DD(1) = 1.0D0 - FD4;
DD(2) = FD3 + FD5;
DD(3) = DD(2) - FD4;
DD(4) = DD(2) - FD1;
DD(5) = -FD5;
DD(6) = DD(3) - FD2;


% Compute phase and frequency of the given tidal constituent
 FREQ=0;
 PHASE=0;
for I = 1:6
    FREQ =  FREQ + IDOOD(I)*DD(I);
    PHASE = PHASE + IDOOD(I)*D(I);
end

% * Adjust phases so that they fall in the positive range 0 to 360
PHASE = mod(PHASE,360);
if(PHASE < 0)
    PHASE = PHASE + 360;
end

